from django.core.validators import RegexValidator
from django.db import models


class BusinessIdentifierField(models.CharField):
    default_validators = [
        RegexValidator(
            regex=r"^[1-9a-zA-Z_.-]+$",
            message="Business ID must be alphanumeric, dots, hyphens, and underscores only.",
        )
    ]

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("max_length", 255)
        super().__init__(*args, **kwargs)


class OneSentenceDescriptionField(models.TextField):
    """
    This field is used to store the description of a business event.
    """

    default_validators = [
        RegexValidator(
            regex=r"^[A-Za-z][^\.:\?!]*[\.:?!]$",
            message="This field should contain only one sentence.",
        )
    ]



# Generated by Django 4.2.6 on 2023-10-28 23:13

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

SENTINEL_NULL_TRANSACTION_ID = getattr(settings, "SENTINEL_NULL_TRANSACTION_ID", 1)


def add_sentinel(apps, schema_editor):
    Transaction = apps.get_model("django_anchor_modeling", "Transaction")
    Transaction.objects.get_or_create(pk=SENTINEL_NULL_TRANSACTION_ID)


def remove_sentinel(apps, schema_editor):
    Transaction = apps.get_model("django_anchor_modeling", "Transaction")
    Transaction.objects.filter(pk=SENTINEL_NULL_TRANSACTION_ID).delete()


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Transaction",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="HistorizedModelMap",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "active_model",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="historized_map_active",
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "historized_model",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="historized_map_historized",
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
        migrations.RunPython(add_sentinel, remove_sentinel),
    ]



# Generated by Django 4.2.6 on 2023-10-24 10:04

from django.db import migrations, models
import django.db.models.deletion
import django_anchor_modeling.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('metadata', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BusinessToDataFieldMap',
            fields=[
                ('created', models.DateTimeField(auto_now_add=True)),
                ('id', django_anchor_modeling.fields.BusinessIdentifierField(max_length=255, primary_key=True, serialize=False)),
                ('description', models.TextField()),
                ('map', models.JSONField(default=dict)),
                ('metadata', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='metadata.datachange')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



DATAVIEWER_CANONICAL_FIELD_DATA_MAP = {
    "id": {"field": "id", "model": "BusinessEvent"},
    "name": {
        "type": "prefetch_related",
        "field": "value",
        "model": "HistorizedBusinessEventName",
        "order_by": "-from_epoch",
        "related_name": "name",
    },
}
import json

DATAVIEWER_CANONICAL_FIELD_DATA_MAP_IN_JSON = json.dumps(
    DATAVIEWER_CANONICAL_FIELD_DATA_MAP
)



def get_knot_choice_value(instance: object):
    """
    Given any instance of a model, return the knot choice value
    format is

    APP__CLASS
    """
    app_label = instance._meta.app_label
    class_name = instance.__class__.__name__

    return f"{app_label}__{class_name}"



# Generated by Django 4.2.6 on 2023-10-29 01:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dataviewer', '0002_businesstodatafieldmap_main_model_class'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='businesstodatafieldmap',
            name='metadata',
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-28 19:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dataviewer', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='businesstodatafieldmap',
            name='main_model_class',
            field=models.CharField(max_length=255, null=True),
        ),
    ]
